KeyRef

Struct KeyRef 

pub struct KeyRef<'r> { /* private fields */ }
Available on crate feature allow-non-fips only.
A reference to a Key, allowing for efficient key handling without ownership.

Implementations

Source

impl<'r> KeyRef<'r>

pub const fn new(inner: &'r [u8; 32]) -> Self

Creates a new KeyRef from a reference to a 32-byte array.

Arguments
  • inner - A reference to a 32-byte array containing the key material.
Example
use wolf_crypto::mac::poly1305::{Key, KeyRef};

let key_bytes = [2u8; 32];
let key = Key::new(key_bytes,);
let key_ref: KeyRef = key.as_ref();
pub const fn copy(&self) -> Key

Creates a copy of the key as a Key.

Returns

A new Key instance containing the same key material.

Example
use wolf_crypto::mac::poly1305::{Key, KeyRef};

let key_ref: KeyRef = (&[7u8; 32]).into();
let owned_key = key_ref.copy();