pub struct AadSlice<'s> { /* private fields */ }Expand description
Represents Additional Authenticated Data (AAD) Slice.
Implementations
Sourceimpl<'s> AadSlice<'s>
impl<'s> AadSlice<'s>
Sourcepub const fn size(&self) -> Option<u32>
pub const fn size(&self) -> Option<u32>
Returns the length of the underlying slice if it can be represented as a u32.
Note
AadSlice::EMPTY will always return Some(0).
Sourcepub const fn valid_size(&self) -> bool
pub const fn valid_size(&self) -> bool
Returns true if the length of the underlying slice can be safely represented as a u32.