Module buf

Module buf 

Types for Keys, IVs, and Generally Sensitive Bytes

Structs

InvalidSize
Error returned when the provided slice is not the expected length.
Iv
Represents an IV / Nonce with the size: U16.
Nonce
Represents an IV / Nonce with the size: U12.
Nonce16
Represents an IV / Nonce with the size: U16.
U12
Represents a 12 byte IV size.
U16
Represents a 16 byte IV size.

Traits

ByteArray
A trait for types that represent byte arrays.
GenericIv
A trait for types that can be used as generic initialization vectors.
IvSize
A trait for types that represent initialization vector (IV) sizes.