Enum wolf_crypto::buf::Buf

source ·
pub enum Buf {
    B128([u8; 128]),
    B256([u8; 256]),
    B384([u8; 384]),
    B512([u8; 512]),
}

Variants



B128([u8; 128])



B256([u8; 256])



B384([u8; 384])



B512([u8; 512])

Implementations

source

impl Buf

source

pub const fn is_sensitive(&self) -> bool

Returns false

source

pub const fn is_128(&self) -> bool

Checks if the buffer is the 128-byte variant.

source

pub const fn is_256(&self) -> bool

Checks if the buffer is the 256-byte variant.

source

pub const fn is_384(&self) -> bool

Checks if the buffer is the 384-byte variant.

source

pub const fn is_512(&self) -> bool

Checks if the buffer is the 512-byte variant.

source

pub const fn len(&self) -> usize

Returns the length of the buffer in bytes.

source

pub const fn as_slice(&self) -> &[u8]

Returns a reference to the buffer as a slice.

source

pub fn as_mut_slice(&mut self) -> &mut [u8]

Returns a mutable reference to the buffer as a slice.