ByteArray

Trait ByteArray 

pub trait ByteArray {
    type Target;

    // Required methods
    fn capacity(&self) -> usize;
    fn slice(&self) -> &[u8] ;
    fn mut_slice(&mut self) -> &mut [u8] ;

    // Provided method
    fn zero(&mut self) { ... }
}
A trait for types that represent byte arrays.

Required Associated Types

type Target

The target type of the byte array.

Required Methods

fn capacity(&self) -> usize

Returns the readable capacity of the byte array.

fn slice(&self) -> &[u8]

Returns a reference to the byte array as a slice.

fn mut_slice(&mut self) -> &mut [u8]

Returns a mutable reference to the byte array as a slice.

Provided Methods

fn zero(&mut self)

Zeros out the contents of the byte array.

Implementations on Foreign Types

Source

impl ByteArray for Vec<u8>

Available on crate feature alloc only.
Source

type Target = Vec<u8>

Source

fn capacity(&self) -> usize

Source

fn slice(&self) -> &[u8]

Source

fn mut_slice(&mut self) -> &mut [u8]

Source

impl<const N: usize> ByteArray for [u8; N]

Source

type Target = [u8; N]

Source

fn capacity(&self) -> usize

Source

fn slice(&self) -> &[u8]

Source

fn mut_slice(&mut self) -> &mut [u8]

Implementors