pub trait GenericIv {
type Size: IvSize;
// Required method
fn as_slice(&self) -> &[u8] ⓘ;
}Expand description
A trait for types that can be used as generic initialization vectors.
pub trait GenericIv {
type Size: IvSize;
// Required method
fn as_slice(&self) -> &[u8] ⓘ;
}A trait for types that can be used as generic initialization vectors.