Writer

Struct Writer 

pub struct Writer<W, const CHUNK: usize> { /* private fields */ }
Available on crate features allow-non-fips and std only.
A wrapper for any implementor of std::io::Write.

Writer implements std::io::Write and takes a child which also implements this trait. This type can wrap any writer, and ensure all data passed to said writer is encrypted.

Implementations

Source

impl<W, const CHUNK: usize> Writer<W, CHUNK>

pub const fn new(chacha: ChaCha20<Streaming>, writer: W) -> Result<Self, W>

Creates a new Writer instance.

Arguments
  • chacha - The ChaCha20 instance in streaming mode.
  • writer - The underlying writer.
Returns

A new Writer instance.

Errors

If the size of CHUNK is greater than u32::MAX

pub fn finish(self) -> ChaCha20<NeedsIv>

Finishes the streaming encryption and returns to the NeedsIv state.

Returns

A ChaCha20 instance in the NeedsIv state.

Methods from Deref<Target = ChaCha20<Streaming>>

pub fn decrypt_into( &mut self, cipher: &[u8], plain: &mut [u8], ) -> Result<(), Unspecified>

Decrypts the ciphertext into the output buffer.

Arguments
  • cipher - The ciphertext to decrypt.
  • plain - The buffer to store the decrypted data.
Errors
  • If the length of cipher is greater than u32::MAX.
  • If the length of cipher is greater than the length of plain.
pub fn decrypt_in_place<'io>( &mut self, in_out: &'io mut [u8], ) -> Result<&'io [u8], Unspecified>

Decrypts the ciphertext in-place.

Arguments
  • in_out - The plaintext to decrypt in place.
Errors

If the length of in_out is greater than u32::MAX.

Returns

The in_out argument, decrypted, for convenience. This can be ignored.

pub fn decrypt_into_sized<const I: usize, const O: usize>( &mut self, cipher: &[u8; I], plain: &mut [u8; O], ) -> Result<(), Unspecified>

Decrypts the ciphertext into the output buffer with compile-time safety checks.

Arguments
  • cipher - The ciphertext array to decrypt.
  • plain - The array to store the decrypted data.
Errors
  • If the length of cipher is greater than u32::MAX.
  • If the length of cipher is greater than the length of plain.
pub fn decrypt_in_place_sized<'io, const C: usize>( &mut self, in_out: &'io mut [u8; C], ) -> Result<&'io [u8; C], Unspecified>

Decrypts the ciphertext in-place with compile-time safety checks.

Arguments
  • in_out - The ciphertext to decrypt in-place.
Returns

ChaCha20 instance in the NeedsIv state.

Errors
  • The length of in_out was greater than u32::MAX.
pub fn decrypt_into_exact<const C: usize>( &mut self, cipher: &[u8; C], plain: &mut [u8; C], ) -> Result<(), Unspecified>

Decrypts the ciphertext into the output buffer with exact sizes.

Arguments
  • cipher - The ciphertext array to decrypt.
  • plain - The array to store the decrypted data.
Errors

If C (the length of cipher and plain) is greater than u32::MAX.

pub fn decrypt(&mut self, cipher: &[u8]) -> Result<Vec<u8>, Unspecified>

Available on crate feature alloc only.

Decrypts the ciphertext and returns the plaintext as a vector.

Arguments
  • cipher - The ciphertext to decrypt.
Errors

If the length of cipher is greater than u32::MAX.

Returns

A newly allocated buffer, the same length as cipher, containing the decrypted plaintext.

pub fn decrypt_exact<const O: usize>( &mut self, cipher: &[u8; O], ) -> Result<[u8; O], Unspecified>

Decrypts the ciphertext array and returns the plaintext array.

Arguments
  • cipher - The ciphertext array to decrypt.
Errors

If O (the length of cipher) is greater than u32::MAX.

pub fn encrypt_into( &mut self, plain: &[u8], cipher: &mut [u8], ) -> Result<(), Unspecified>

Encrypts the input into the output buffer in streaming mode.

Arguments
  • plain - The input to encrypt.
  • cipher - The buffer to store the encrypted data.
Errors
  • The length of cipher is less than the length of plain.
  • The length of plain is greater than u32::MAX.
pub fn encrypt_into_sized<const I: usize, const O: usize>( &mut self, plain: &[u8; I], cipher: &mut [u8; O], ) -> Result<(), Unspecified>

Encrypts the input into the output buffer in streaming mode with compile-time size checking.

Arguments
  • plain - The input array to encrypt.
  • cipher - The array to store the encrypted data.
Errors
  • The length of plain was greater than u32::MAX.
  • The length of cipher was less than the length of plain.
pub fn encrypt_into_exact<const C: usize>( &mut self, input: &[u8; C], output: &mut [u8; C], ) -> Result<(), Unspecified>

Encrypts the input into the output buffer in streaming mode with exact sizes.

Type Parameters
  • C - The size of both the input and output arrays.
Arguments
  • input - The input array to encrypt.
  • output - The array to store the encrypted data.
Errors

C (the length of input and output) was greater than u32::MAX.