Module kdf

Module kdf 

Key Derivation Functions

Modules

pbkdf
The Password Based Key Derivation Function 1 and 2
salt
Salt requirement marker types.

Structs

FipsPbkdf2
A wrapper type enforcing FIPS compliance for PBKDF2 operations.
InsecureKey
Represents a key associated with the desired hashing function which can be insecure.
Iters
The number of iterations for PBKDF.
KeySlice
Represents a key for the associated hashing algorithm which has a length greater than or equal to the length of the hash function’s digest.
Md5allow-non-fips
The MD5 Hash Function Marker Type.
Shaallow-non-fips
The SHA-1 Hash Function Marker Type.
Sha3_224
The SHA3-224 Hash Function Marker Type.
Sha3_256
The SHA3-256 Hash Function Marker Type.
Sha3_384
The SHA3-384 Hash Function Marker Type.
Sha3_512
The SHA3-512 Hash Function Marker Type.
Sha224
The SHA224 Hash Function Marker Type.
Sha256
The SHA256 Hash Function Marker Type.
Sha384
The SHA384 Hash Function Marker Type.
Sha512
The SHA512 Hash Function Marker Type.

Traits

Salt
Represents a salt value used in key derivation functions (KDFs).

Functions

hkdfallow-non-fips
Performs HKDF and returns the result as a fixed-size array.
hkdf_intoallow-non-fips
Performs HKDF and writes the result into the provided output buffer.
pbkdf1allow-non-fips
Performs PBKDF1 and returns the result as a fixed-size array.
pbkdf2
Performs PBKDF2 and returns the result as a fixed-size array.
pbkdf1_intoallow-non-fips
Performs PBKDF1 and writes the result into the provided out_key buffer.
pbkdf2_into
Performs PBKDF2 and writes the result into the provided out_key buffer.

Type Aliases

DynSaltSliceallow-non-fips
A salt::Slice which will be a FipsSaltSlice if the allow-non-fips feature is disabled, or a SaltSlice if the feature is enabled.
FipsSaltSlice
A salt::Slice which meets the FIPS requirement (128 bits) in length.
MaybeSaltSlice
A salt::Slice which may be left empty.
SaltSlice
A salt::Slice which must not be empty.