Slice

Struct Slice 

pub struct Slice<'s, SZ> { /* private fields */ }
A Salt with runtime flexibility.

The Salt trait, with its associated constraints, is implemented for most common types which meet the marker constraint for pure compile-time checks. However, this can be limiting, this type moves these compile-time checks to runtime.

Implementations

Source

impl<'s> Slice<'s, Empty>

pub const fn new(slice: &'s [u8]) -> Result<Self, Infallible>

Create a new SaltSlice instance.

Arguments
  • slice - The Salt (if any) to use.
Errors

This is infallible, the only reason this returns a result is to keep this new implementation in sync with other Salt constraints (this being the weakest / most permissive constraint).

Source

impl<'s> Slice<'s, NonEmpty>

pub const fn new(slice: &'s [u8]) -> Result<Self, InvalidSize>

Create a new SaltSlice instance.

Arguments
  • slice - The Salt, which must be non-empty.
Errors

This requires the provided slice to be non-empty, this is the second-weakest constraint, and only leveraged with allow-non-fips enabled. In general, for KDFs such as PBKDF it is strongly recommended to use at least a 128 bit (16 byte) salt generated from a valid CSPRNG.

Source

impl<'s> Slice<'s, Min16>

pub const fn new(slice: &'s [u8]) -> Result<Self, InvalidSize>

Create a new SaltSlice instance.

Arguments
  • slice - The Salt, which must be at least 128 bits (16 bytes).
Errors

This requires that the provided slice is at least 128 bits (16 bytes), this is the strongest constraint as it enforces this best practice (as well as FIPS requirement). Regardless if the interface requires this constraint it is strongly recommended to use a 128 bit salt generated from a valid CSPRNG.