pub struct KeySlice<'k, SZ: KeySz> { /* private fields */ }Expand description
Represents a key for the associated hashing algorithm which has a length greater than or equal to the length of the hash function’s digest.
Implementations