pub trait Salt<SZ: MinSize>: Sealed { }Expand description
Represents a salt value used in key derivation functions (KDFs).
This is only implemented for HKDF, as other KDFs require salts (such as the PBKDF
family), and for FIPS compliance, as per NIST SP 800-132, Section 5.1, The Salt (S)
this salt must be at least 128 bits from a valid CSPRNG.
Salt is a critical component in KDFs, used to:
- Increase the complexity of the derived key
- Mitigate rainbow table attacks
- Ensure unique keys even when the same input is used multiple times
Salts may be optional, this depends on the MinSz type.