Module hmac

Module hmac 

Keyed Hash Message Authentication Codes HMAC.

Modules

algo
Collection of marker types and their associated keys which denote the hashing function.

Structs

Digest
Utility wrapper around the final HMAC hash.
HexDigest
Utility wrapper around the hex-encoded final HMAC hash.
Hmac
Hashed-Based Message Authentication Codes HMAC.
KeySlice
Represents a key for the associated hashing algorithm which has a length greater than or equal to the length of the hash function’s digest.
Md5allow-non-fips
The MD5 Hash Function Marker Type.
Shaallow-non-fips
The SHA-1 Hash Function Marker Type.
Sha3_224
The SHA3-224 Hash Function Marker Type.
Sha3_256
The SHA3-256 Hash Function Marker Type.
Sha3_384
The SHA3-384 Hash Function Marker Type.
Sha3_512
The SHA3-512 Hash Function Marker Type.
Sha224
The SHA224 Hash Function Marker Type.
Sha256
The SHA256 Hash Function Marker Type.
Sha384
The SHA384 Hash Function Marker Type.
Sha512
The SHA512 Hash Function Marker Type.