KeySlice

Struct KeySlice 

pub struct KeySlice<'k, SZ: KeySz> { /* private fields */ }
Represents a key for the associated hashing algorithm which has a length greater than or equal to the length of the hash function’s digest.

Implementations

Source

impl<'k, SZ: KeySz> KeySlice<'k, SZ>

pub fn new(slice: &'k [u8]) -> Result<Self, InvalidSize>

Try creating a new KeySlice instance.

Errors
  • If the length of the slice is less than the SZ::size.
  • If the length of the slice is greater than u32::MAX.