Md5

Struct Md5 

pub struct Md5;
Available on crate feature allow-non-fips only.
The MD5 Hash Function Marker Type.

MD5 should be considered cryptographically broken and unsuitable for further use. Collision attacks against MD5 are both practical and trivial, theoretical attacks against MD5 have been found.

MD5 is included in this library for legacy reasons only.