Key

Struct Key 

pub struct Key { /* private fields */ }
Available on crate feature allow-non-fips only.
Represents a 32-byte secret key for Poly1305 and ChaCha20Poly1305.

This struct ensures that the key material is securely managed and zeroed from memory when dropped.

Implementations

Source

impl Key

pub const fn new(inner: [u8; 32]) -> Self

Creates a new Key from a 32-byte array.

Arguments
  • inner - A 32-byte array containing the key material.
Example
use wolf_crypto::mac::poly1305::Key;

let key = Key::new([0u8; 32]);
pub const fn as_ref(&self) -> KeyRef<'_>

Returns a reference to the key as a KeyRef.

Returns
  • KeyRef - A reference to the key material.
Example
use wolf_crypto::mac::poly1305::{Key, KeyRef};

let key = Key::new([0u8; 32]);
let key_ref: KeyRef = key.as_ref();