StreamPoly1305Aead

Struct StreamPoly1305Aead 

pub struct StreamPoly1305Aead { /* private fields */ }
Available on crate feature allow-non-fips only.
Represents an ongoing streaming MAC computation, allowing incremental updates.

This uses the TLS AEAD padding scheme.

Example

use wolf_crypto::{mac::{Poly1305, poly1305::Key}, aead::Tag};

let key: Key = [42u8; 32].into();
let tag = Poly1305::new(key.as_ref())
    .aead_padding()
    .update(b"chunk1")?
    .update(b"chunk2")?
    .update(b"chunk3")?
    .finalize();

Implementations

Source

impl StreamPoly1305Aead

pub fn update(self, input: &[u8]) -> Result<Self, Self>

Updates the streaming MAC computation with additional input.

Arguments
  • input - A byte slice representing the additional data to include.
Errors
  • The length of the input was greater than u32::MAX.
  • The total length that has been processed is greater than u32::MAX,
Example
use wolf_crypto::{mac::{Poly1305, poly1305::Key}, aead::Tag};

let key: Key = [42u8; 32].into();

let tag = Poly1305::new(key.as_ref())
    .aead_padding()
    .update(b"chunk1")?
    .update(b"chunk2")?
    .update(b"chunk3")?
    .finalize();
pub fn finalize(self) -> Tag

Finalizes the streaming MAC computation and returns the resulting Tag.

Returns

The associated authentication Tag.

Example
use wolf_crypto::{mac::{Poly1305, poly1305::Key}, aead::Tag};

let key: Key = [42u8; 32].into();

let tag = Poly1305::new(key.as_ref())
    .aead_padding()
    .update(b"chunk1")?
    .update(b"chunk2")?
    .update(b"chunk3")?
    .finalize();