pub struct Tag { /* private fields */ }Expand description
Represents the authentication tag for AEADs
Implementations
Sourceimpl Tag
impl Tag
Sourcepub const fn new_zeroed() -> Self
pub const fn new_zeroed() -> Self
Creates a new Tag instance filled with zeros.
This is typically used to create a tag buffer that will be filled by an encryption operation.
Returns
A new Tag instance with all bytes set to zero.
Example
use wolf_crypto::aead::Tag;
let tag = Tag::new_zeroed();
assert_eq!(tag.as_slice(), &[0u8; 16]);Trait Implementations
Sourceimpl<T> PartialEq<&T> for Tag
impl<T> PartialEq<&T> for Tag
Sourceimpl<T> PartialEq<&mut T> for Tag
impl<T> PartialEq<&mut T> for Tag
impl Copy for Tag