Struct wolf_crypto::aes::Tag
source · pub struct Tag { /* private fields */ }Expand description
Represents the authentication tag produced by AES-GCM encryption.
Implementations
sourceimpl Tag
impl Tag
sourcepub const fn new_zeroed() -> Self
pub const fn new_zeroed() -> Self
Creates a new Tag instance filled with zeros.
This is typically used to create a tag buffer that will be filled by an encryption operation.
Returns
A new Tag instance with all bytes set to zero.
Example
use wolf_crypto::aes::gcm::Tag;
let tag = Tag::new_zeroed();
assert_eq!(tag.as_slice(), &[0u8; 16]);Trait Implementations