wolf_crypto

Module buf

source
Types for Keys, IVs, and Generally Sensitive Bytes

Structs

  • Error returned when the provided slice is not the expected length.
  • Represents an IV / Nonce with the size: U16.
  • Represents an IV / Nonce with the size: U12.
  • Represents an IV / Nonce with the size: U16.
  • Represents a 12 byte IV size.
  • Represents a 16 byte IV size.

Traits

  • A trait for types that represent byte arrays.
  • A trait for types that can be used as generic initialization vectors.
  • A trait for types that represent initialization vector (IV) sizes.