pub struct Writer<W, const CHUNK: usize> { /* private fields */ }Available on crate features
allow-non-fips and std only.Expand description
A wrapper for any implementor of std::io::Write.
Writer implements std::io::Write and takes a child which also implements this trait.
This type can wrap any writer, and ensure all data passed to said writer is encrypted.
Implementations
Methods from Deref<Target = ChaCha20<Streaming>>
Sourcepub fn decrypt_into(
&mut self,
cipher: &[u8],
plain: &mut [u8],
) -> Result<(), Unspecified>
pub fn decrypt_into( &mut self, cipher: &[u8], plain: &mut [u8], ) -> Result<(), Unspecified>
Sourcepub fn decrypt_in_place<'io>(
&mut self,
in_out: &'io mut [u8],
) -> Result<&'io [u8], Unspecified>
pub fn decrypt_in_place<'io>( &mut self, in_out: &'io mut [u8], ) -> Result<&'io [u8], Unspecified>
Sourcepub fn decrypt_into_sized<const I: usize, const O: usize>(
&mut self,
cipher: &[u8; I],
plain: &mut [u8; O],
) -> Result<(), Unspecified>
pub fn decrypt_into_sized<const I: usize, const O: usize>( &mut self, cipher: &[u8; I], plain: &mut [u8; O], ) -> Result<(), Unspecified>
Sourcepub fn decrypt_in_place_sized<'io, const C: usize>(
&mut self,
in_out: &'io mut [u8; C],
) -> Result<&'io [u8; C], Unspecified>
pub fn decrypt_in_place_sized<'io, const C: usize>( &mut self, in_out: &'io mut [u8; C], ) -> Result<&'io [u8; C], Unspecified>
Sourcepub fn decrypt_into_exact<const C: usize>(
&mut self,
cipher: &[u8; C],
plain: &mut [u8; C],
) -> Result<(), Unspecified>
pub fn decrypt_into_exact<const C: usize>( &mut self, cipher: &[u8; C], plain: &mut [u8; C], ) -> Result<(), Unspecified>
Sourcepub fn decrypt(&mut self, cipher: &[u8]) -> Result<Vec<u8>, Unspecified>
Available on crate feature alloc only.
pub fn decrypt(&mut self, cipher: &[u8]) -> Result<Vec<u8>, Unspecified>
alloc only.Sourcepub fn decrypt_exact<const O: usize>(
&mut self,
cipher: &[u8; O],
) -> Result<[u8; O], Unspecified>
pub fn decrypt_exact<const O: usize>( &mut self, cipher: &[u8; O], ) -> Result<[u8; O], Unspecified>
Sourcepub fn encrypt_into(
&mut self,
plain: &[u8],
cipher: &mut [u8],
) -> Result<(), Unspecified>
pub fn encrypt_into( &mut self, plain: &[u8], cipher: &mut [u8], ) -> Result<(), Unspecified>
Sourcepub fn encrypt_into_sized<const I: usize, const O: usize>(
&mut self,
plain: &[u8; I],
cipher: &mut [u8; O],
) -> Result<(), Unspecified>
pub fn encrypt_into_sized<const I: usize, const O: usize>( &mut self, plain: &[u8; I], cipher: &mut [u8; O], ) -> Result<(), Unspecified>
Sourcepub fn encrypt_into_exact<const C: usize>(
&mut self,
input: &[u8; C],
output: &mut [u8; C],
) -> Result<(), Unspecified>
pub fn encrypt_into_exact<const C: usize>( &mut self, input: &[u8; C], output: &mut [u8; C], ) -> Result<(), Unspecified>
Encrypts the input into the output buffer in streaming mode with exact sizes.
Type Parameters
C- The size of both the input and output arrays.
Arguments
input- The input array to encrypt.output- The array to store the encrypted data.
Errors
C (the length of input and output) was greater than u32::MAX.