Struct wolf_crypto::chacha::Writer

source ·
pub struct Writer<W, const CHUNK: usize> { /* private fields */ }
Available on crate feature std only.
A wrapper for any implementor of std::io::Write.

Writer implements std::io::Write and takes a child which also implements this trait. This type can wrap any writer, and ensure all data passed to said writer is encrypted.

Implementations

source

impl<W, const CHUNK: usize> Writer<W, CHUNK>

source

pub const fn new(chacha: ChaCha20<Streaming>, writer: W) -> Self

Creates a new Writer instance.

Arguments
  • chacha - The ChaCha20 instance in streaming mode.
  • writer - The underlying writer.
Returns

A new Writer instance.

source

pub fn finish(self) -> ChaCha20<NeedsIv>

Finishes the streaming encryption and returns to the NeedsIv state.

Returns

A ChaCha20 instance in the NeedsIv state.

Methods from Deref<Target = ChaCha20<Streaming>>

source

pub fn decrypt_into(&mut self, cipher: &[u8], output: &mut [u8]) -> Res

Decrypts the ciphertext into the output buffer.

Arguments
  • cipher - The ciphertext to decrypt.
  • output - The buffer to store the decrypted data.
Returns

A Res indicating the success or failure of the operation.

source

pub fn decrypt_into_sized<const I: usize, const O: usize>( &mut self, cipher: &[u8; I], output: &mut [u8; O], ) -> Res

Decrypts the ciphertext into the output buffer with compile-time size checking.

Type Parameters
  • I - The size of the ciphertext array.
  • O - The size of the output array.
Arguments
  • cipher - The ciphertext array to decrypt.
  • output - The array to store the decrypted data.
Returns

A Res indicating the success or failure of the operation.

source

pub fn decrypt_into_exact<const C: usize>( &mut self, cipher: &[u8; C], output: &mut [u8; C], ) -> Res

Decrypts the ciphertext into the output buffer with exact sizes.

Type Parameters
  • C - The size of both the ciphertext and output arrays.
Arguments
  • cipher - The ciphertext array to decrypt.
  • output - The array to store the decrypted data.
Returns

A Res indicating the success or failure of the operation.

source

pub fn decrypt(&mut self, cipher: &[u8]) -> Result<Vec<u8>, Unspecified>

Available on crate feature alloc only.

Decrypts the ciphertext and returns the plaintext as a vector.

Arguments
  • cipher - The ciphertext to decrypt.
Returns

A Result containing either the decrypted plaintext as a vector on success, or an Unspecified error on failure.

source

pub fn decrypt_exact<const O: usize>( &mut self, cipher: &[u8; O], ) -> Result<[u8; O], Unspecified>

Decrypts the ciphertext array and returns the plaintext array.

Type Parameters
  • O - The size of the ciphertext and plaintext arrays.
Arguments
  • cipher - The ciphertext array to decrypt.
Returns

A Result containing either the decrypted plaintext array on success, or an Unspecified error on failure.

source

pub fn encrypt_into(&mut self, input: &[u8], output: &mut [u8]) -> Res

Encrypts the input into the output buffer in streaming mode.

Arguments
  • input - The input to encrypt.
  • output - The buffer to store the encrypted data.
Returns

A Res indicating the success or failure of the operation.

source

pub fn encrypt_into_sized<const I: usize, const O: usize>( &mut self, input: &[u8; I], output: &mut [u8; O], ) -> Res

Encrypts the input into the output buffer in streaming mode with compile-time size checking.

Type Parameters
  • I - The size of the input array.
  • O - The size of the output array.
Arguments
  • input - The input array to encrypt.
  • output - The array to store the encrypted data.
Returns

A Res indicating the success or failure of the operation.

source

pub fn encrypt_into_exact<const C: usize>( &mut self, input: &[u8; C], output: &mut [u8; C], ) -> Res

Encrypts the input into the output buffer in streaming mode with exact sizes.

Type Parameters
  • C - The size of both the input and output arrays.
Arguments
  • input - The input array to encrypt.
  • output - The array to store the encrypted data.
Returns

A Res indicating the success or failure of the operation.