Key Derivation Functions
Modules
Structs
- Fips
Pbkdf2 - A wrapper type enforcing FIPS compliance for PBKDF2 operations.
- Insecure
Key - Represents a key associated with the desired hashing function which can be insecure.
- Iters
- The number of iterations for PBKDF.
- KeySlice
- Represents a key for the associated hashing algorithm which has a length greater than or equal to the length of the hash function’s digest.
- Md5
allow-non-fips - The
MD5Hash Function Marker Type. - Sha
allow-non-fips - The
SHA-1Hash Function Marker Type. - Sha3_
224 - The
SHA3-224Hash Function Marker Type. - Sha3_
256 - The
SHA3-256Hash Function Marker Type. - Sha3_
384 - The
SHA3-384Hash Function Marker Type. - Sha3_
512 - The
SHA3-512Hash Function Marker Type. - Sha224
- The
SHA224Hash Function Marker Type. - Sha256
- The
SHA256Hash Function Marker Type. - Sha384
- The
SHA384Hash Function Marker Type. - Sha512
- The
SHA512Hash Function Marker Type.
Traits
- Salt
- Represents a salt value used in key derivation functions (KDFs).
Functions
- hkdf
allow-non-fips - Performs HKDF and returns the result as a fixed-size array.
- hkdf_
into allow-non-fips - Performs HKDF and writes the result into the provided output buffer.
- pbkdf1
allow-non-fips - Performs PBKDF1 and returns the result as a fixed-size array.
- pbkdf2
- Performs PBKDF2 and returns the result as a fixed-size array.
- pbkdf1_
into allow-non-fips - Performs PBKDF1 and writes the result into the provided
out_keybuffer. - pbkdf2_
into - Performs PBKDF2 and writes the result into the provided
out_keybuffer.
Type Aliases
- DynSalt
Slice allow-non-fips - A
salt::Slicewhich will be aFipsSaltSliceif theallow-non-fipsfeature is disabled, or aSaltSliceif the feature is enabled. - Fips
Salt Slice - A
salt::Slicewhich meets the FIPS requirement (128 bits) in length. - Maybe
Salt Slice - A
salt::Slicewhich may be left empty. - Salt
Slice - A
salt::Slicewhich must not be empty.