pub struct Slice<'s, SZ> { /* private fields */ }Expand description
Implementations
Sourceimpl<'s> Slice<'s, NonEmpty>
impl<'s> Slice<'s, NonEmpty>
Sourcepub const fn new(slice: &'s [u8]) -> Result<Self, InvalidSize>
pub const fn new(slice: &'s [u8]) -> Result<Self, InvalidSize>
Create a new SaltSlice instance.
Arguments
slice- TheSalt, which must be non-empty.
Errors
This requires the provided slice to be non-empty, this is the second-weakest
constraint, and only leveraged with allow-non-fips enabled. In general, for KDFs such
as PBKDF it is strongly recommended to use at least a 128 bit (16 byte) salt
generated from a valid CSPRNG.
Sourceimpl<'s> Slice<'s, Min16>
impl<'s> Slice<'s, Min16>
Sourcepub const fn new(slice: &'s [u8]) -> Result<Self, InvalidSize>
pub const fn new(slice: &'s [u8]) -> Result<Self, InvalidSize>
Create a new SaltSlice instance.
Arguments
slice- TheSalt, which must be at least 128 bits (16 bytes).
Errors
This requires that the provided slice is at least 128 bits (16 bytes), this is the
strongest constraint as it enforces this best practice (as well as FIPS requirement).
Regardless if the interface requires this constraint it is strongly recommended to
use a 128 bit salt generated from a valid CSPRNG.