pub struct Sha;Available on crate feature
allow-non-fips only.Expand description
The SHA-1 Hash Function Marker Type.
The SHA-1 algorithm is included in this library for legacy reasons only. It is cryptographically broken and should not be used for any security-critical or modern applications, especially digital signatures or certificate validation.
The U.S. National Institute of Standards and Technology (NIST) has officially deprecated SHA-1 for all digital signature use cases as of 2011. As of 2022, NIST recommends transitioning all applications from SHA-1 to SHA-2 or SHA-3 family of hash functions.