Keyed Hash Message Authentication Codes HMAC.
Modules
- algo
- Collection of marker types and their associated keys which denote the hashing function.
Structs
- Digest
- Utility wrapper around the final
HMAChash. - HexDigest
- Utility wrapper around the hex-encoded final
HMAChash. - Hmac
- Hashed-Based Message Authentication Codes
HMAC. - KeySlice
- Represents a key for the associated hashing algorithm which has a length greater than or equal to the length of the hash function’s digest.
- Md5
allow-non-fips - The
MD5Hash Function Marker Type. - Sha
allow-non-fips - The
SHA-1Hash Function Marker Type. - Sha3_
224 - The
SHA3-224Hash Function Marker Type. - Sha3_
256 - The
SHA3-256Hash Function Marker Type. - Sha3_
384 - The
SHA3-384Hash Function Marker Type. - Sha3_
512 - The
SHA3-512Hash Function Marker Type. - Sha224
- The
SHA224Hash Function Marker Type. - Sha256
- The
SHA256Hash Function Marker Type. - Sha384
- The
SHA384Hash Function Marker Type. - Sha512
- The
SHA512Hash Function Marker Type.