pub trait GenericKey: Sealed {
type Size: KeySz;
// Required methods
fn size(&self) -> u32;
fn cleanup(self);
}Expand description
Represents a valid key for the associated hashing algorithm..
pub trait GenericKey: Sealed {
type Size: KeySz;
// Required methods
fn size(&self) -> u32;
fn cleanup(self);
}Represents a valid key for the associated hashing algorithm..