pub trait GenericKey: Sealed { }Available on crate feature
allow-non-fips only.Expand description
A sealed trait for generic key types used in Poly1305.
This trait is sealed and cannot be implemented outside of this crate.
pub trait GenericKey: Sealed { }allow-non-fips only.A sealed trait for generic key types used in Poly1305.
This trait is sealed and cannot be implemented outside of this crate.