wolf_crypto::mac::poly1305

Trait GenericKey

source
pub trait GenericKey: Sealed { }
Available on crate feature allow-non-fips only.
A sealed trait for generic key types used in Poly1305.

This trait is sealed and cannot be implemented outside of this crate.

Implementors